@@ -169,14 +169,14 @@ def UpdateForFlowControl(ii):
 		ii.flowControl = FlowControl.CMOV
 		return
 
-	# Should I include SYSCALL ?
 	pairs = [
 		(["INT", "INT1", "INT 3", "INTO", "UD2"], FlowControl.INT),
 		(["CALL", "CALL FAR"], FlowControl.CALL),
 		(["RET", "IRET", "RETF"], FlowControl.RET),
 		(["SYSCALL", "SYSENTER", "SYSRET", "SYSEXIT"], FlowControl.SYS),
 		(["JMP", "JMP FAR"], FlowControl.UNC_BRANCH),
-		(["JCXZ", "JO", "JNO", "JB", "JAE", "JZ", "JNZ", "JBE", "JA", "JS", "JNS", "JP", "JNP", "JL", "JGE", "JLE", "JG", "LOOP", "LOOPZ", "LOOPNZ"], FlowControl.CND_BRANCH)
+		(["JCXZ", "JO", "JNO", "JB", "JAE", "JZ", "JNZ", "JBE", "JA", "JS", "JNS", "JP", "JNP", "JL", "JGE", "JLE", "JG", "LOOP", "LOOPZ", "LOOPNZ"], FlowControl.CND_BRANCH),
+		(["HLT"], FlowControl.HLT)
 	]
 	ii.flowControl = 0
 	for p in pairs:
@@ -234,9 +234,10 @@ def UpdatePrivilegedInstruction(opcodeIds, ii):
 		# IO Sensitive Instructions, mostly allowed by ring0 only.
 		"IN", "INS", "OUT", "OUTS", "CLI", "STI", "IRET"
 	]
+	ii.privileged = False
 	for i in enumerate(ii.mnemonics):
-		if (i[1] in privileged) or IsPrivilegedMov(ii):	
-			opcodeIds[i[0]] |= 0x8000
+		if (i[1] in privileged) or IsPrivilegedMov(ii):
+			ii.privileged = True
 
 def SetInstructionAffectedFlags(ii, flagsTuple):
 	""" Helper routine to set the m/t/u flags for an instruction info. """
@@ -457,8 +458,9 @@ def FormatInstruction(ii, mnemonicsIds):
 	if flagsIndex >= 256:
 		raise Exception("FlagsIndex exceeded its 8 bits. Change flags of _InstInfo to be uint16!")
 
+	privileged = 0x8000 if ii.privileged else 0
 	# InstSharedInfo:
-	sharedInfo = (flagsIndex, ops[1], ops[0], (ii.classType << 3) | ii.flowControl, ii.modifiedFlags, ii.testedFlags, ii.undefinedFlags)
+	sharedInfo = (flagsIndex, ops[1], ops[0], ii.modifiedFlags, ii.testedFlags, ii.undefinedFlags, (ii.classType << 8) | ii.flowControl | privileged)
 	if sharedInfo not in sharedInfoDict:
 		sharedInfoDict[sharedInfo] = len(sharedInfoDict)
 	# Get the shared-info-index.
@@ -528,7 +530,7 @@ def CreateTables(db):
 	So instead of generating the following old data layout:
 	{&II_00, &II_01, &II_02, NULL, NULL, &II_05, &II_06, NULL}
 	(Actually the old layout is a bit more complicated and consumes another byte for indicating the type of node.)
-	
+
 	Anyways, we can generate the follow table:
 	{1, 2, 3, 0, 0, 4, 5, 0}
 	This time the table is in bytes, a byte is enough to index 256 instructions (which is a Full sized table).
@@ -581,7 +583,7 @@ def CreateTables(db):
 
 	:!:NOTE:!: You MUST iterate a table with GenBlock wrapper, otherwise you might NOT get all instructions from the DB!
 		   Refer to x86db.py-class GenBlock for more information. """
-	indexShift = 13 # According to InstNode in instructions.h.
+	typeShift = 13 # According to InstNode in instructions.h.
 	InstInfos = []
 	InstInfosEx = []
 	InstructionsTree = []
@@ -602,13 +604,17 @@ def CreateTables(db):
 				if isExtended:
 					InstInfosEx.append(formattedII)
 					index = len(InstInfosEx) - 1
-					InstructionsTree.append((NodeType.INFOEX << indexShift | index, i.tag))
+					InstructionsTree.append((NodeType.INFOEX << typeShift | index, i.tag))
 				else:
 					InstInfos.append(formattedII)
 					index = len(InstInfos) - 1
-					InstructionsTree.append((NodeType.INFO << indexShift | index, i.tag))
+					nodeType = NodeType.INFO << typeShift
+					# LEA, ARPL and NOP are manually treated in diStorm, so give them a different type.
+					if (i.OL == OpcodeLength.OL_1 and i.pos[0] in [0x90, 0x8d, 0x63]):
+						nodeType = NodeType.INFO_TREAT << typeShift
+					InstructionsTree.append((nodeType | index, i.tag))
 			elif isinstance(i, x86db.InstructionsTable):
-				InstructionsTree.append(((i.type << indexShift) | nextTableIndex, i.tag))
+				InstructionsTree.append(((i.type << typeShift) | nextTableIndex, i.tag))
 				nextTableIndex += i.size # This assumes we walk on the instructions tables in BFS order!
 			else:
 				# False indicates this entry points nothing.
